# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Docker VM implementation.
"""

from gns3.vm import VM
from gns3.node import Node
from gns3.ports.port import Port
from gns3.ports.ethernet_port import EthernetPort
from gns3.nios.nio_generic_ethernet import NIOGenericEthernet
from gns3.nios.nio_linux_ethernet import NIOLinuxEthernet
from gns3.nios.nio_udp import NIOUDP
from .settings import DOCKER_CONTAINER_SETTINGS

import logging
import re
log = logging.getLogger(__name__)


class DockerVM(VM):
    """
    Docker Image.

    :param module: parent module for this node
    :param server: GNS3 server instance
    """
    URL_PREFIX = "docker"

    def __init__(self, module, server, project):
        super().__init__(module, server, project)

        log.info("Docker image instance is being created")
        self._settings = {
            "name": "",
            "image": "",
            "adapters": DOCKER_CONTAINER_SETTINGS["adapters"],
            "start_command": DOCKER_CONTAINER_SETTINGS["start_command"],
            "environment": DOCKER_CONTAINER_SETTINGS["environment"],
            "console": None,
            "aux": None,
            "console_type": DOCKER_CONTAINER_SETTINGS["console_type"],
            "console_resolution": DOCKER_CONTAINER_SETTINGS["console_resolution"],
            "console_http_port": DOCKER_CONTAINER_SETTINGS["console_http_port"],
            "console_http_path": DOCKER_CONTAINER_SETTINGS["console_http_path"]
        }

    def _addAdapters(self, adapters):
        """Adds adapters.

        :param adapters: number of adapters
        """
        for adapter_number in range(0, adapters):
            adapter_name = EthernetPort.longNameType() + str(adapter_number)
            short_name = EthernetPort.shortNameType() + str(adapter_number)
            new_port = EthernetPort(adapter_name)
            new_port.setShortName(short_name)
            new_port.setAdapterNumber(adapter_number)
            new_port.setPortNumber(0)
            new_port.setHotPluggable(False)
            new_port.setPacketCaptureSupported(True)
            self._ports.append(new_port)
            log.debug("Adapter {} has been added".format(adapter_name))

    def setup(self, image, name=None, base_name=None, vm_id=None, additional_settings={}, default_name_format="{name}-{0}"):
        """Sets up this Docker container.

        :param image: image name
        :param name: optional name
        :param additional_settings: additional settings for this VM
        """
        # let's create a unique name if none has been chosen
        if not name:
            name = self.allocateName(default_name_format.replace('{name}', base_name))

        if not name:
            self.error_signal.emit(self.id(), "could not allocate a name for this container")
            return

        self.setName(name)
        self._settings["name"] = name
        self._settings["image"] = image
        params = {
            "name": name,
            "image": image,
            "adapters": self._settings["adapters"]
        }
        if vm_id:
            params["vm_id"] = vm_id
        params.update(additional_settings)

        self.httpPost("/docker/vms", self._setupCallback, body=params, timeout=None)

    def _setupCallback(self, result, error=False, **kwargs):
        """Callback for Docker container setup.

        :param result: server response
        :param error: indicates an error (boolean)
        """
        if not super()._setupCallback(result, error=error, **kwargs):
            return

        self._addAdapters(self._settings.get("adapters", 0))

        if self._loading:
            self.loaded_signal.emit()
        else:
            self.setInitialized(True)
            log.info(
                "Docker container {} has been created".format(self.name()))
            self.created_signal.emit(self.id())
            self._module.addNode(self)

    def _updateCallback(self, result, error=False, **kwargs):
        """
        Callback for update.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return

        updated = False
        nb_adapters_changed = False
        for name, value in result.items():
            if name in self._settings and self._settings[name] != value:
                log.info("{}: updating {} from '{}' to '{}'".format(self.name(), name, self._settings[name], value))
                updated = True
                if name == "name":
                    # update the node name
                    self.updateAllocatedName(value)
                if name == "adapters":
                    nb_adapters_changed = True
                self._settings[name] = value

        if nb_adapters_changed:
            log.debug("number of adapters has changed to {}".format(self._settings["adapters"]))
            # TODO: dynamically add/remove adapters
            self._ports.clear()
            self._addAdapters(self._settings["adapters"])

        if updated:
            log.info("Docker VM {} has been updated".format(self.name()))
            self.updated_signal.emit()

    def update(self, new_settings):
        """
        Updates the settings for this VPCS device.

        :param new_settings: settings dictionary
        """

        if "name" in new_settings and new_settings["name"] != self.name() and self.hasAllocatedName(new_settings["name"]):
            self.error_signal.emit(self.id(), 'Name "{}" is already used by another node'.format(new_settings["name"]))
            return

        params = {}
        for name, value in new_settings.items():
            if name in self._settings and self._settings[name] != value:
                params[name] = value

        log.debug("{} is updating settings: {}".format(self.name(), params))
        self.httpPut("/docker/vms/{vm_id}".format(project_id=self._project.id(), vm_id=self._vm_id), self._updateCallback, body=params)

    def suspend(self):
        """Suspends this Docker container."""
        if self.status() == Node.suspended:
            log.debug("{} is already suspended".format(self.name()))
            return
        log.debug("{} is being suspended".format(self.name()))
        self.httpPost("/docker/vms/{id}/suspend".format(
            id=self._vm_id), self._suspendCallback)

    def _suspendCallback(self, result, error=False, **kwargs):
        """Callback for container suspend.

        :param result: server response
        :param error: indicates an error (boolean)
        """
        if error:
            log.error("error while suspending {}: {}".format(
                self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has suspended".format(self.name()))
            self.setStatus(Node.suspended)
            for port in self._ports:
                # set ports as suspended
                port.setStatus(Port.suspended)
            self.suspended_signal.emit()

    def dump(self):
        """
        Returns a representation of this Docker VM instance.
        (to be saved in a topology file).

        :returns: representation of the node (dictionary)
        """
        docker = super().dump()
        docker["id"] = self.id()
        docker["vm_id"] = self._vm_id

        # add the properties
        for name, value in self._settings.items():
            if value is not None and value != "":
                docker["properties"][name] = value

        # add the ports
        if self._ports:
            ports = docker["ports"] = []
            for port in self._ports:
                ports.append(port.dump())
        return docker

    def info(self):
        """Returns information about this Docker container.

        :returns: formated string
        :rtype: string
        """
        if self.status() == Node.started:
            state = "started"
        else:
            state = "stopped"

        info = """Docker container {name} is {state}
  Node ID is {id}, server's Docker container ID is {vm_id}
""".format(name=self.name(),
           id=self.id(),
           vm_id=self._vm_id,
           state=state
           )

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(
                    port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(
                    port_name=port.name(),
                    port_description=port.description())

        return info + port_info

    def load(self, node_info):
        """
        Loads a Docker representation
        (from a topology file).

        :param node_info: representation of the node (dictionary)
        """

        super().load(node_info)

        settings = node_info["properties"]
        name = settings.pop("name")
        image = settings.pop("image")
        vm_id = node_info["vm_id"]
        log.info("Docker container {} is loading".format(name))
        self.setup(image, name=name, vm_id=vm_id, additional_settings=settings)

    def _updatePortSettings(self):
        """
        Updates port settings when loading a topology.
        """

        self.loaded_signal.disconnect(self._updatePortSettings)

        # assign the correct names and IDs to the ports
        if "ports" in self._node_info:
            ports = self._node_info["ports"]
            for topology_port in ports:
                for port in self._ports:
                    adapter_number = topology_port.get("adapter_number")
                    if adapter_number == port.adapterNumber():
                        port.setName(topology_port["name"])
                        port.setId(topology_port["id"])

        # now we can set the node as initialized and trigger the created signal
        self.setInitialized(True)
        log.info("Docker container {} has been loaded".format(self.name()))
        self.created_signal.emit(self.id())
        self._module.addNode(self)
        self._loading = False
        self._node_info = None

    def name(self):
        """
        Returns the name of this Docker container.

        :returns: name (string)
        """
        return self._settings["name"]

    def settings(self):
        """
        Returns all settings of this Docker container.

        :returns: settings
        :rtype: dict
        """
        return self._settings

    def ports(self):
        """
        Returns all the ports for this Docker VM instance.

        :returns: list of Port instances
        """
        return self._ports

    def console(self):
        """
        Returns the console port for this Docker VM instance.

        :returns: port (integer)
        """
        return self._settings["console"]

    def consoleHttpPath(self):
        """
        Returns the path of the web ui

        :returns: string
        """
        return self._settings["console_http_path"]

    def auxConsole(self):
        """
        Returns the console port for this Docker VM instance.

        :returns: port (integer)
        """
        return self._settings["aux"]

    def configPage(self):
        """Returns the configuration page widget to be used by the node configurator.

        :returns: QWidget object
        """
        from .pages.docker_vm_configuration_page import DockerVMConfigurationPage
        return DockerVMConfigurationPage

    @staticmethod
    def defaultSymbol():
        """Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """
        return ":/symbols/docker_guest.svg"

    def networkInterfacesPath(self):
        """
        Return path of the /etc/network/interfaces
        """
        return "/project-files/docker/{}/etc/network/interfaces".format(self._vm_id)

    @staticmethod
    def symbolName():
        return "Docker container"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node category (integer)
        """
        return [Node.end_devices]

    def __str__(self):
        return "Docker container"





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/server_preferences_page.ui'
#
# Created: Thu May  5 18:26:47 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ServerPreferencesPageWidget(object):
    def setupUi(self, ServerPreferencesPageWidget):
        ServerPreferencesPageWidget.setObjectName("ServerPreferencesPageWidget")
        ServerPreferencesPageWidget.resize(526, 596)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ServerPreferencesPageWidget.sizePolicy().hasHeightForWidth())
        ServerPreferencesPageWidget.setSizePolicy(sizePolicy)
        ServerPreferencesPageWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ServerPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiServerPreferenceTabWidget = QtWidgets.QTabWidget(ServerPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerPreferenceTabWidget.sizePolicy().hasHeightForWidth())
        self.uiServerPreferenceTabWidget.setSizePolicy(sizePolicy)
        self.uiServerPreferenceTabWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.uiServerPreferenceTabWidget.setObjectName("uiServerPreferenceTabWidget")
        self.uiLocalTabWidget = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalTabWidget.sizePolicy().hasHeightForWidth())
        self.uiLocalTabWidget.setSizePolicy(sizePolicy)
        self.uiLocalTabWidget.setObjectName("uiLocalTabWidget")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.uiLocalTabWidget)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.uiGeneralSettingsGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiGeneralSettingsGroupBox.setObjectName("uiGeneralSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGeneralSettingsGroupBox)
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.gridLayout.setObjectName("gridLayout")
        self.uiLocalServerPathLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPathLabel.setObjectName("uiLocalServerPathLabel")
        self.gridLayout.addWidget(self.uiLocalServerPathLabel, 1, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiLocalServerPathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPathLineEdit.setObjectName("uiLocalServerPathLineEdit")
        self.horizontalLayout.addWidget(self.uiLocalServerPathLineEdit)
        self.uiLocalServerToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiLocalServerToolButton.setObjectName("uiLocalServerToolButton")
        self.horizontalLayout.addWidget(self.uiLocalServerToolButton)
        self.gridLayout.addLayout(self.horizontalLayout, 2, 0, 1, 1)
        self.uiUbridgePathLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiUbridgePathLabel.setObjectName("uiUbridgePathLabel")
        self.gridLayout.addWidget(self.uiUbridgePathLabel, 3, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiUbridgePathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsGroupBox)
        self.uiUbridgePathLineEdit.setObjectName("uiUbridgePathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiUbridgePathLineEdit)
        self.uiUbridgeToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsGroupBox)
        self.uiUbridgeToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiUbridgeToolButton.setObjectName("uiUbridgeToolButton")
        self.horizontalLayout_5.addWidget(self.uiUbridgeToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_5, 4, 0, 1, 1)
        self.uiLocalServerHostLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerHostLabel.setObjectName("uiLocalServerHostLabel")
        self.gridLayout.addWidget(self.uiLocalServerHostLabel, 5, 0, 1, 1)
        self.uiLocalServerHostComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerHostComboBox.setObjectName("uiLocalServerHostComboBox")
        self.gridLayout.addWidget(self.uiLocalServerHostComboBox, 6, 0, 1, 1)
        self.uiLocalServerPortLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPortLabel.setObjectName("uiLocalServerPortLabel")
        self.gridLayout.addWidget(self.uiLocalServerPortLabel, 7, 0, 1, 1)
        self.uiLocalServerPortSpinBox = QtWidgets.QSpinBox(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPortSpinBox.setSuffix(" TCP")
        self.uiLocalServerPortSpinBox.setMaximum(65535)
        self.uiLocalServerPortSpinBox.setProperty("value", 3080)
        self.uiLocalServerPortSpinBox.setObjectName("uiLocalServerPortSpinBox")
        self.gridLayout.addWidget(self.uiLocalServerPortSpinBox, 8, 0, 1, 1)
        self.uiConsoleConnectionsToAnyIPCheckBox = QtWidgets.QCheckBox(self.uiGeneralSettingsGroupBox)
        self.uiConsoleConnectionsToAnyIPCheckBox.setObjectName("uiConsoleConnectionsToAnyIPCheckBox")
        self.gridLayout.addWidget(self.uiConsoleConnectionsToAnyIPCheckBox, 10, 0, 1, 1)
        self.uiLocalServerAuthCheckBox = QtWidgets.QCheckBox(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerAuthCheckBox.setObjectName("uiLocalServerAuthCheckBox")
        self.gridLayout.addWidget(self.uiLocalServerAuthCheckBox, 9, 0, 1, 1)
        self.gridLayout_6.addWidget(self.uiGeneralSettingsGroupBox, 1, 0, 1, 2)
        self.uiLocalServerAutoStartCheckBox = QtWidgets.QCheckBox(self.uiLocalTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalServerAutoStartCheckBox.sizePolicy().hasHeightForWidth())
        self.uiLocalServerAutoStartCheckBox.setSizePolicy(sizePolicy)
        self.uiLocalServerAutoStartCheckBox.setMinimumSize(QtCore.QSize(0, 0))
        self.uiLocalServerAutoStartCheckBox.setChecked(False)
        self.uiLocalServerAutoStartCheckBox.setObjectName("uiLocalServerAutoStartCheckBox")
        self.gridLayout_6.addWidget(self.uiLocalServerAutoStartCheckBox, 0, 0, 1, 1)
        self.uiConsolePortRangeGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiConsolePortRangeGroupBox.setObjectName("uiConsolePortRangeGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiConsolePortRangeGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiConsoleStartPortSpinBox = QtWidgets.QSpinBox(self.uiConsolePortRangeGroupBox)
        self.uiConsoleStartPortSpinBox.setSuffix(" TCP")
        self.uiConsoleStartPortSpinBox.setMaximum(65535)
        self.uiConsoleStartPortSpinBox.setProperty("value", 2000)
        self.uiConsoleStartPortSpinBox.setObjectName("uiConsoleStartPortSpinBox")
        self.gridLayout_4.addWidget(self.uiConsoleStartPortSpinBox, 0, 0, 1, 1)
        self.uiConsolePortRangeLabel = QtWidgets.QLabel(self.uiConsolePortRangeGroupBox)
        self.uiConsolePortRangeLabel.setObjectName("uiConsolePortRangeLabel")
        self.gridLayout_4.addWidget(self.uiConsolePortRangeLabel, 0, 1, 1, 1)
        self.uiConsoleEndPortSpinBox = QtWidgets.QSpinBox(self.uiConsolePortRangeGroupBox)
        self.uiConsoleEndPortSpinBox.setSuffix(" TCP")
        self.uiConsoleEndPortSpinBox.setMaximum(65535)
        self.uiConsoleEndPortSpinBox.setProperty("value", 5000)
        self.uiConsoleEndPortSpinBox.setObjectName("uiConsoleEndPortSpinBox")
        self.gridLayout_4.addWidget(self.uiConsoleEndPortSpinBox, 0, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(225, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 0, 3, 1, 1)
        self.gridLayout_6.addWidget(self.uiConsolePortRangeGroupBox, 2, 0, 1, 2)
        self.uiUDPPortRangeGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiUDPPortRangeGroupBox.setObjectName("uiUDPPortRangeGroupBox")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.uiUDPPortRangeGroupBox)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.uiUDPStartPortSpinBox = QtWidgets.QSpinBox(self.uiUDPPortRangeGroupBox)
        self.uiUDPStartPortSpinBox.setSuffix(" UDP")
        self.uiUDPStartPortSpinBox.setMaximum(65535)
        self.uiUDPStartPortSpinBox.setProperty("value", 10000)
        self.uiUDPStartPortSpinBox.setObjectName("uiUDPStartPortSpinBox")
        self.horizontalLayout_8.addWidget(self.uiUDPStartPortSpinBox)
        self.uiUDPPortRangeLabel = QtWidgets.QLabel(self.uiUDPPortRangeGroupBox)
        self.uiUDPPortRangeLabel.setObjectName("uiUDPPortRangeLabel")
        self.horizontalLayout_8.addWidget(self.uiUDPPortRangeLabel)
        self.uiUDPEndPortSpinBox = QtWidgets.QSpinBox(self.uiUDPPortRangeGroupBox)
        self.uiUDPEndPortSpinBox.setSuffix(" UDP")
        self.uiUDPEndPortSpinBox.setMaximum(65535)
        self.uiUDPEndPortSpinBox.setProperty("value", 20000)
        self.uiUDPEndPortSpinBox.setObjectName("uiUDPEndPortSpinBox")
        self.horizontalLayout_8.addWidget(self.uiUDPEndPortSpinBox)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.gridLayout_6.addWidget(self.uiUDPPortRangeGroupBox, 3, 0, 1, 2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem2, 4, 0, 1, 1)
        self.uiServerPreferenceTabWidget.addTab(self.uiLocalTabWidget, "")
        self.uiGNS3VMTabWidget = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiGNS3VMTabWidget.sizePolicy().hasHeightForWidth())
        self.uiGNS3VMTabWidget.setSizePolicy(sizePolicy)
        self.uiGNS3VMTabWidget.setObjectName("uiGNS3VMTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiGNS3VMTabWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiEnableVMCheckBox = QtWidgets.QCheckBox(self.uiGNS3VMTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEnableVMCheckBox.sizePolicy().hasHeightForWidth())
        self.uiEnableVMCheckBox.setSizePolicy(sizePolicy)
        self.uiEnableVMCheckBox.setMinimumSize(QtCore.QSize(0, 0))
        self.uiEnableVMCheckBox.setChecked(False)
        self.uiEnableVMCheckBox.setObjectName("uiEnableVMCheckBox")
        self.verticalLayout.addWidget(self.uiEnableVMCheckBox)
        self.uiVirtualizationGroupBox = QtWidgets.QGroupBox(self.uiGNS3VMTabWidget)
        self.uiVirtualizationGroupBox.setObjectName("uiVirtualizationGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiVirtualizationGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiVmwareRadioButton = QtWidgets.QRadioButton(self.uiVirtualizationGroupBox)
        self.uiVmwareRadioButton.setChecked(True)
        self.uiVmwareRadioButton.setObjectName("uiVmwareRadioButton")
        self.buttonGroup = QtWidgets.QButtonGroup(ServerPreferencesPageWidget)
        self.buttonGroup.setObjectName("buttonGroup")
        self.buttonGroup.addButton(self.uiVmwareRadioButton)
        self.gridLayout_7.addWidget(self.uiVmwareRadioButton, 0, 0, 1, 1)
        self.uiVirtualBoxRadioButton = QtWidgets.QRadioButton(self.uiVirtualizationGroupBox)
        self.uiVirtualBoxRadioButton.setObjectName("uiVirtualBoxRadioButton")
        self.buttonGroup.addButton(self.uiVirtualBoxRadioButton)
        self.gridLayout_7.addWidget(self.uiVirtualBoxRadioButton, 0, 1, 1, 1)
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiVirtualizationGroupBox)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.buttonGroup.addButton(self.uiRemoteRadioButton)
        self.gridLayout_7.addWidget(self.uiRemoteRadioButton, 0, 2, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(166, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem3, 0, 3, 1, 1)
        self.verticalLayout.addWidget(self.uiVirtualizationGroupBox)
        self.uiLocalGNS3VMSettingsGroupBox = QtWidgets.QGroupBox(self.uiGNS3VMTabWidget)
        self.uiLocalGNS3VMSettingsGroupBox.setObjectName("uiLocalGNS3VMSettingsGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiLocalGNS3VMSettingsGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiVMNameLabel = QtWidgets.QLabel(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiVMNameLabel.setObjectName("uiVMNameLabel")
        self.gridLayout_2.addWidget(self.uiVMNameLabel, 0, 0, 1, 1)
        self.uiVMListComboBox = QtWidgets.QComboBox(self.uiLocalGNS3VMSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMListComboBox.sizePolicy().hasHeightForWidth())
        self.uiVMListComboBox.setSizePolicy(sizePolicy)
        self.uiVMListComboBox.setObjectName("uiVMListComboBox")
        self.gridLayout_2.addWidget(self.uiVMListComboBox, 0, 1, 1, 1)
        self.uiRefreshPushButton = QtWidgets.QPushButton(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiRefreshPushButton.setObjectName("uiRefreshPushButton")
        self.gridLayout_2.addWidget(self.uiRefreshPushButton, 0, 2, 1, 1)
        self.uiHeadlessCheckBox = QtWidgets.QCheckBox(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiHeadlessCheckBox.setObjectName("uiHeadlessCheckBox")
        self.gridLayout_2.addWidget(self.uiHeadlessCheckBox, 1, 0, 1, 2)
        self.uiShutdownCheckBox = QtWidgets.QCheckBox(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiShutdownCheckBox.setObjectName("uiShutdownCheckBox")
        self.gridLayout_2.addWidget(self.uiShutdownCheckBox, 2, 0, 1, 2)
        self.uiAdjustLocalServerIPCheckBox = QtWidgets.QCheckBox(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiAdjustLocalServerIPCheckBox.setObjectName("uiAdjustLocalServerIPCheckBox")
        self.gridLayout_2.addWidget(self.uiAdjustLocalServerIPCheckBox, 3, 0, 1, 2)
        self.verticalLayout.addWidget(self.uiLocalGNS3VMSettingsGroupBox)
        self.uiRemoteGNS3VMSettingsGroupBox = QtWidgets.QGroupBox(self.uiGNS3VMTabWidget)
        self.uiRemoteGNS3VMSettingsGroupBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.uiRemoteGNS3VMSettingsGroupBox.setFlat(False)
        self.uiRemoteGNS3VMSettingsGroupBox.setObjectName("uiRemoteGNS3VMSettingsGroupBox")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.uiRemoteGNS3VMSettingsGroupBox)
        self.horizontalLayout_4.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label = QtWidgets.QLabel(self.uiRemoteGNS3VMSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout_4.addWidget(self.label)
        self.uiServersComboBox = QtWidgets.QComboBox(self.uiRemoteGNS3VMSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiServersComboBox.setSizePolicy(sizePolicy)
        self.uiServersComboBox.setObjectName("uiServersComboBox")
        self.horizontalLayout_4.addWidget(self.uiServersComboBox)
        self.uiAddServerPushButton = QtWidgets.QPushButton(self.uiRemoteGNS3VMSettingsGroupBox)
        self.uiAddServerPushButton.setObjectName("uiAddServerPushButton")
        self.horizontalLayout_4.addWidget(self.uiAddServerPushButton)
        self.verticalLayout.addWidget(self.uiRemoteGNS3VMSettingsGroupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)
        self.uiServerPreferenceTabWidget.addTab(self.uiGNS3VMTabWidget, "")
        self.uiRemoteTabWidget = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteTabWidget.sizePolicy().hasHeightForWidth())
        self.uiRemoteTabWidget.setSizePolicy(sizePolicy)
        self.uiRemoteTabWidget.setObjectName("uiRemoteTabWidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiRemoteTabWidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiRemoteServersTreeWidget = QtWidgets.QTreeWidget(self.uiRemoteTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersTreeWidget.setSizePolicy(sizePolicy)
        self.uiRemoteServersTreeWidget.setColumnCount(4)
        self.uiRemoteServersTreeWidget.setObjectName("uiRemoteServersTreeWidget")
        self.uiRemoteServersTreeWidget.headerItem().setText(0, "Protocol")
        self.uiRemoteServersTreeWidget.headerItem().setText(1, "Host")
        self.uiRemoteServersTreeWidget.headerItem().setText(2, "Port")
        self.gridLayout_5.addWidget(self.uiRemoteServersTreeWidget, 0, 0, 1, 2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiAddRemoteServerPushButton = QtWidgets.QPushButton(self.uiRemoteTabWidget)
        self.uiAddRemoteServerPushButton.setObjectName("uiAddRemoteServerPushButton")
        self.horizontalLayout_3.addWidget(self.uiAddRemoteServerPushButton)
        self.uiDeleteRemoteServerPushButton = QtWidgets.QPushButton(self.uiRemoteTabWidget)
        self.uiDeleteRemoteServerPushButton.setEnabled(False)
        self.uiDeleteRemoteServerPushButton.setObjectName("uiDeleteRemoteServerPushButton")
        self.horizontalLayout_3.addWidget(self.uiDeleteRemoteServerPushButton)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem5)
        self.gridLayout_5.addLayout(self.horizontalLayout_3, 1, 0, 1, 2)
        spacerItem6 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem6, 2, 0, 1, 1)
        self.uiServerPreferenceTabWidget.addTab(self.uiRemoteTabWidget, "")
        self.verticalLayout_2.addWidget(self.uiServerPreferenceTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem7)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(ServerPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(ServerPreferencesPageWidget)
        self.uiServerPreferenceTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ServerPreferencesPageWidget)
        ServerPreferencesPageWidget.setTabOrder(self.uiServerPreferenceTabWidget, self.uiLocalServerAutoStartCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerAutoStartCheckBox, self.uiLocalServerPathLineEdit)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerPathLineEdit, self.uiLocalServerToolButton)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerToolButton, self.uiUbridgePathLineEdit)
        ServerPreferencesPageWidget.setTabOrder(self.uiUbridgePathLineEdit, self.uiUbridgeToolButton)
        ServerPreferencesPageWidget.setTabOrder(self.uiUbridgeToolButton, self.uiLocalServerHostComboBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerHostComboBox, self.uiLocalServerPortSpinBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerPortSpinBox, self.uiLocalServerAuthCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiLocalServerAuthCheckBox, self.uiConsoleConnectionsToAnyIPCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiConsoleConnectionsToAnyIPCheckBox, self.uiConsoleStartPortSpinBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiConsoleStartPortSpinBox, self.uiConsoleEndPortSpinBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiConsoleEndPortSpinBox, self.uiUDPStartPortSpinBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiUDPStartPortSpinBox, self.uiUDPEndPortSpinBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiUDPEndPortSpinBox, self.uiRestoreDefaultsPushButton)
        ServerPreferencesPageWidget.setTabOrder(self.uiRestoreDefaultsPushButton, self.uiEnableVMCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiEnableVMCheckBox, self.uiHeadlessCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiHeadlessCheckBox, self.uiShutdownCheckBox)
        ServerPreferencesPageWidget.setTabOrder(self.uiShutdownCheckBox, self.uiRemoteServersTreeWidget)
        ServerPreferencesPageWidget.setTabOrder(self.uiRemoteServersTreeWidget, self.uiAddRemoteServerPushButton)
        ServerPreferencesPageWidget.setTabOrder(self.uiAddRemoteServerPushButton, self.uiDeleteRemoteServerPushButton)

    def retranslateUi(self, ServerPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        ServerPreferencesPageWidget.setWindowTitle(_translate("ServerPreferencesPageWidget", "Server"))
        self.uiGeneralSettingsGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "General settings"))
        self.uiLocalServerPathLabel.setText(_translate("ServerPreferencesPageWidget", "Server path:"))
        self.uiLocalServerToolButton.setText(_translate("ServerPreferencesPageWidget", "&Browse..."))
        self.uiUbridgePathLabel.setText(_translate("ServerPreferencesPageWidget", "Ubridge path:"))
        self.uiUbridgeToolButton.setText(_translate("ServerPreferencesPageWidget", "&Browse..."))
        self.uiLocalServerHostLabel.setText(_translate("ServerPreferencesPageWidget", "Host binding:"))
        self.uiLocalServerPortLabel.setText(_translate("ServerPreferencesPageWidget", "Port:"))
        self.uiConsoleConnectionsToAnyIPCheckBox.setText(_translate("ServerPreferencesPageWidget", "Allow console connections to any local IP address"))
        self.uiLocalServerAuthCheckBox.setText(_translate("ServerPreferencesPageWidget", "Protect server with password (recommended)"))
        self.uiLocalServerAutoStartCheckBox.setText(_translate("ServerPreferencesPageWidget", "Enable local server"))
        self.uiConsolePortRangeGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "Console port range (5900 => 6000 is shared with VNC)"))
        self.uiConsolePortRangeLabel.setText(_translate("ServerPreferencesPageWidget", "to"))
        self.uiUDPPortRangeGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "UDP tunneling port range"))
        self.uiUDPPortRangeLabel.setText(_translate("ServerPreferencesPageWidget", "to"))
        self.uiServerPreferenceTabWidget.setTabText(self.uiServerPreferenceTabWidget.indexOf(self.uiLocalTabWidget), _translate("ServerPreferencesPageWidget", "Local server"))
        self.uiEnableVMCheckBox.setText(_translate("ServerPreferencesPageWidget", "Enable the GNS3 VM"))
        self.uiVirtualizationGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "Virtualization software"))
        self.uiVmwareRadioButton.setToolTip(_translate("ServerPreferencesPageWidget", "VMware is recommended to run Qemu based appliances (required for KVM)."))
        self.uiVmwareRadioButton.setText(_translate("ServerPreferencesPageWidget", "VMware"))
        self.uiVirtualBoxRadioButton.setToolTip(_translate("ServerPreferencesPageWidget", "Use VirtualBox if you intend to run Dynamips, IOU or VPCS appliances."))
        self.uiVirtualBoxRadioButton.setText(_translate("ServerPreferencesPageWidget", "VirtualBox"))
        self.uiRemoteRadioButton.setToolTip(_translate("ServerPreferencesPageWidget", "Use remote if your VM is run on a remote server (e.g. ESXi or dedicated server)."))
        self.uiRemoteRadioButton.setText(_translate("ServerPreferencesPageWidget", "Remote"))
        self.uiLocalGNS3VMSettingsGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "Local GNS3 VM"))
        self.uiVMNameLabel.setText(_translate("ServerPreferencesPageWidget", "VM name:"))
        self.uiRefreshPushButton.setText(_translate("ServerPreferencesPageWidget", "&Refresh"))
        self.uiHeadlessCheckBox.setText(_translate("ServerPreferencesPageWidget", "Start VM in headless mode"))
        self.uiShutdownCheckBox.setText(_translate("ServerPreferencesPageWidget", "ACPI shutdown VM when closing GNS3"))
        self.uiAdjustLocalServerIPCheckBox.setText(_translate("ServerPreferencesPageWidget", "Adjust the local server IP to be in the same subnet"))
        self.uiRemoteGNS3VMSettingsGroupBox.setTitle(_translate("ServerPreferencesPageWidget", "Remote GNS3 VM"))
        self.label.setText(_translate("ServerPreferencesPageWidget", "Server:"))
        self.uiAddServerPushButton.setText(_translate("ServerPreferencesPageWidget", "Add server"))
        self.uiServerPreferenceTabWidget.setTabText(self.uiServerPreferenceTabWidget.indexOf(self.uiGNS3VMTabWidget), _translate("ServerPreferencesPageWidget", "GNS3 VM server"))
        self.uiRemoteServersTreeWidget.headerItem().setText(3, _translate("ServerPreferencesPageWidget", "User"))
        self.uiAddRemoteServerPushButton.setText(_translate("ServerPreferencesPageWidget", "&Add"))
        self.uiDeleteRemoteServerPushButton.setText(_translate("ServerPreferencesPageWidget", "&Delete"))
        self.uiServerPreferenceTabWidget.setTabText(self.uiServerPreferenceTabWidget.indexOf(self.uiRemoteTabWidget), _translate("ServerPreferencesPageWidget", "Remote servers"))
        self.uiRestoreDefaultsPushButton.setText(_translate("ServerPreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Functions to start VNC console programs.
"""

import sys
import os
import shlex
import subprocess
from .main_window import MainWindow

import logging
log = logging.getLogger(__name__)


def vncConsole(host, port, command):
    """
    Start a VNC console program.

    :param host: host or IP address
    :param port: port number
    """

    # replace the place-holders by the actual values
    command = command.replace("%h", host)
    command = command.replace("%p", str(port))

    try:
        log.info('starting VNC program "{}"'.format(command))
        if sys.platform.startswith("win"):
            # use the string on Windows
            subprocess.Popen(command)
        else:
            # use arguments on other platforms
            args = shlex.split(command)
            subprocess.Popen(args, env=os.environ)
    except (OSError, ValueError, subprocess.SubprocessError) as e:
        log.warning('could not start VNC program "{}": {}'.format(command, e))
        raise



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/dynamips/ui/ios_router_wizard.ui'
#
# Created: Fri Mar 25 14:45:27 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_IOSRouterWizard(object):
    def setupUi(self, IOSRouterWizard):
        IOSRouterWizard.setObjectName("IOSRouterWizard")
        IOSRouterWizard.resize(585, 398)
        IOSRouterWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiServerWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout_3.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.verticalLayout_3.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout_3.addWidget(self.uiLocalRadioButton)
        self.verticalLayout.addWidget(self.uiServerTypeGroupBox)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiRemoteServersGroupBox)
        IOSRouterWizard.addPage(self.uiServerWizardPage)
        self.uiIOSImageWizardPage = QtWidgets.QWizardPage()
        self.uiIOSImageWizardPage.setObjectName("uiIOSImageWizardPage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiIOSImageWizardPage)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiIOSExistingImageRadioButton = QtWidgets.QRadioButton(self.uiIOSImageWizardPage)
        self.uiIOSExistingImageRadioButton.setChecked(True)
        self.uiIOSExistingImageRadioButton.setObjectName("uiIOSExistingImageRadioButton")
        self.horizontalLayout_4.addWidget(self.uiIOSExistingImageRadioButton)
        self.uiIOSNewImageRadioButton = QtWidgets.QRadioButton(self.uiIOSImageWizardPage)
        self.uiIOSNewImageRadioButton.setChecked(False)
        self.uiIOSNewImageRadioButton.setObjectName("uiIOSNewImageRadioButton")
        self.horizontalLayout_4.addWidget(self.uiIOSNewImageRadioButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.uiIOSImageLabel = QtWidgets.QLabel(self.uiIOSImageWizardPage)
        self.uiIOSImageLabel.setObjectName("uiIOSImageLabel")
        self.verticalLayout_2.addWidget(self.uiIOSImageLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOSImageListComboBox = QtWidgets.QComboBox(self.uiIOSImageWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIOSImageListComboBox.sizePolicy().hasHeightForWidth())
        self.uiIOSImageListComboBox.setSizePolicy(sizePolicy)
        self.uiIOSImageListComboBox.setObjectName("uiIOSImageListComboBox")
        self.horizontalLayout_5.addWidget(self.uiIOSImageListComboBox)
        self.uiIOSImageLineEdit = QtWidgets.QLineEdit(self.uiIOSImageWizardPage)
        self.uiIOSImageLineEdit.setObjectName("uiIOSImageLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOSImageLineEdit)
        self.uiIOSImageToolButton = QtWidgets.QToolButton(self.uiIOSImageWizardPage)
        self.uiIOSImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOSImageToolButton.setObjectName("uiIOSImageToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOSImageToolButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        IOSRouterWizard.addPage(self.uiIOSImageWizardPage)
        self.uiNamePlatformWizardPage = QtWidgets.QWizardPage()
        self.uiNamePlatformWizardPage.setObjectName("uiNamePlatformWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiNamePlatformWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiNamePlatformWizardPage)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiChassisComboBox = QtWidgets.QComboBox(self.uiNamePlatformWizardPage)
        self.uiChassisComboBox.setObjectName("uiChassisComboBox")
        self.gridLayout.addWidget(self.uiChassisComboBox, 2, 1, 1, 1)
        self.uiTypeLabel = QtWidgets.QLabel(self.uiNamePlatformWizardPage)
        self.uiTypeLabel.setObjectName("uiTypeLabel")
        self.gridLayout.addWidget(self.uiTypeLabel, 1, 0, 1, 1)
        self.uiNameLabel = QtWidgets.QLabel(self.uiNamePlatformWizardPage)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiPlatformComboBox = QtWidgets.QComboBox(self.uiNamePlatformWizardPage)
        self.uiPlatformComboBox.setObjectName("uiPlatformComboBox")
        self.gridLayout.addWidget(self.uiPlatformComboBox, 1, 1, 1, 1)
        self.uiChassisLabel = QtWidgets.QLabel(self.uiNamePlatformWizardPage)
        self.uiChassisLabel.setObjectName("uiChassisLabel")
        self.gridLayout.addWidget(self.uiChassisLabel, 2, 0, 1, 1)
        self.uiEtherSwitchCheckBox = QtWidgets.QCheckBox(self.uiNamePlatformWizardPage)
        self.uiEtherSwitchCheckBox.setObjectName("uiEtherSwitchCheckBox")
        self.gridLayout.addWidget(self.uiEtherSwitchCheckBox, 3, 0, 1, 2)
        IOSRouterWizard.addPage(self.uiNamePlatformWizardPage)
        self.uiMemoryWizardPage = QtWidgets.QWizardPage()
        self.uiMemoryWizardPage.setObjectName("uiMemoryWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiMemoryWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiRamLabel = QtWidgets.QLabel(self.uiMemoryWizardPage)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_2.addWidget(self.uiRamLabel, 0, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiMemoryWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRamSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRamSpinBox.setSizePolicy(sizePolicy)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setSingleStep(32)
        self.uiRamSpinBox.setProperty("value", 128)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_2.addWidget(self.uiRamSpinBox, 0, 1, 1, 1)
        self.uiTestIOSImagePushButton = QtWidgets.QPushButton(self.uiMemoryWizardPage)
        self.uiTestIOSImagePushButton.setObjectName("uiTestIOSImagePushButton")
        self.gridLayout_2.addWidget(self.uiTestIOSImagePushButton, 0, 2, 1, 1)
        self.label = QtWidgets.QLabel(self.uiMemoryWizardPage)
        self.label.setWordWrap(False)
        self.label.setOpenExternalLinks(True)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 3)
        IOSRouterWizard.addPage(self.uiMemoryWizardPage)
        self.uiNetworkAdaptersWizardPage = QtWidgets.QWizardPage()
        self.uiNetworkAdaptersWizardPage.setObjectName("uiNetworkAdaptersWizardPage")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiNetworkAdaptersWizardPage)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiSlot0Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot0Label.sizePolicy().hasHeightForWidth())
        self.uiSlot0Label.setSizePolicy(sizePolicy)
        self.uiSlot0Label.setObjectName("uiSlot0Label")
        self.gridLayout_4.addWidget(self.uiSlot0Label, 0, 0, 1, 1)
        self.uiSlot0comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot0comboBox.setObjectName("uiSlot0comboBox")
        self.gridLayout_4.addWidget(self.uiSlot0comboBox, 0, 1, 1, 1)
        self.uiSlot1Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot1Label.sizePolicy().hasHeightForWidth())
        self.uiSlot1Label.setSizePolicy(sizePolicy)
        self.uiSlot1Label.setObjectName("uiSlot1Label")
        self.gridLayout_4.addWidget(self.uiSlot1Label, 1, 0, 1, 1)
        self.uiSlot1comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot1comboBox.setObjectName("uiSlot1comboBox")
        self.gridLayout_4.addWidget(self.uiSlot1comboBox, 1, 1, 1, 1)
        self.uiSlot2Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot2Label.sizePolicy().hasHeightForWidth())
        self.uiSlot2Label.setSizePolicy(sizePolicy)
        self.uiSlot2Label.setObjectName("uiSlot2Label")
        self.gridLayout_4.addWidget(self.uiSlot2Label, 2, 0, 1, 1)
        self.uiSlot2comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot2comboBox.setObjectName("uiSlot2comboBox")
        self.gridLayout_4.addWidget(self.uiSlot2comboBox, 2, 1, 1, 1)
        self.uiSlot3Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot3Label.sizePolicy().hasHeightForWidth())
        self.uiSlot3Label.setSizePolicy(sizePolicy)
        self.uiSlot3Label.setObjectName("uiSlot3Label")
        self.gridLayout_4.addWidget(self.uiSlot3Label, 3, 0, 1, 1)
        self.uiSlot3comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot3comboBox.setObjectName("uiSlot3comboBox")
        self.gridLayout_4.addWidget(self.uiSlot3comboBox, 3, 1, 1, 1)
        self.uiSlot4Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot4Label.sizePolicy().hasHeightForWidth())
        self.uiSlot4Label.setSizePolicy(sizePolicy)
        self.uiSlot4Label.setObjectName("uiSlot4Label")
        self.gridLayout_4.addWidget(self.uiSlot4Label, 4, 0, 1, 1)
        self.uiSlot4comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot4comboBox.setObjectName("uiSlot4comboBox")
        self.gridLayout_4.addWidget(self.uiSlot4comboBox, 4, 1, 1, 1)
        self.uiSlot5Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot5Label.sizePolicy().hasHeightForWidth())
        self.uiSlot5Label.setSizePolicy(sizePolicy)
        self.uiSlot5Label.setObjectName("uiSlot5Label")
        self.gridLayout_4.addWidget(self.uiSlot5Label, 5, 0, 1, 1)
        self.uiSlot5comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot5comboBox.setObjectName("uiSlot5comboBox")
        self.gridLayout_4.addWidget(self.uiSlot5comboBox, 5, 1, 1, 1)
        self.uiSlot6Label = QtWidgets.QLabel(self.uiNetworkAdaptersWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot6Label.sizePolicy().hasHeightForWidth())
        self.uiSlot6Label.setSizePolicy(sizePolicy)
        self.uiSlot6Label.setObjectName("uiSlot6Label")
        self.gridLayout_4.addWidget(self.uiSlot6Label, 6, 0, 1, 1)
        self.uiSlot6comboBox = QtWidgets.QComboBox(self.uiNetworkAdaptersWizardPage)
        self.uiSlot6comboBox.setObjectName("uiSlot6comboBox")
        self.gridLayout_4.addWidget(self.uiSlot6comboBox, 6, 1, 1, 1)
        IOSRouterWizard.addPage(self.uiNetworkAdaptersWizardPage)
        self.uiWicWizardPage = QtWidgets.QWizardPage()
        self.uiWicWizardPage.setObjectName("uiWicWizardPage")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiWicWizardPage)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiWic0Label = QtWidgets.QLabel(self.uiWicWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic0Label.sizePolicy().hasHeightForWidth())
        self.uiWic0Label.setSizePolicy(sizePolicy)
        self.uiWic0Label.setObjectName("uiWic0Label")
        self.gridLayout_5.addWidget(self.uiWic0Label, 0, 0, 1, 1)
        self.uiWic0comboBox = QtWidgets.QComboBox(self.uiWicWizardPage)
        self.uiWic0comboBox.setObjectName("uiWic0comboBox")
        self.gridLayout_5.addWidget(self.uiWic0comboBox, 0, 1, 1, 1)
        self.uiWic1Label = QtWidgets.QLabel(self.uiWicWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic1Label.sizePolicy().hasHeightForWidth())
        self.uiWic1Label.setSizePolicy(sizePolicy)
        self.uiWic1Label.setObjectName("uiWic1Label")
        self.gridLayout_5.addWidget(self.uiWic1Label, 1, 0, 1, 1)
        self.uiWic1comboBox = QtWidgets.QComboBox(self.uiWicWizardPage)
        self.uiWic1comboBox.setObjectName("uiWic1comboBox")
        self.gridLayout_5.addWidget(self.uiWic1comboBox, 1, 1, 1, 1)
        self.uiWic2Label = QtWidgets.QLabel(self.uiWicWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic2Label.sizePolicy().hasHeightForWidth())
        self.uiWic2Label.setSizePolicy(sizePolicy)
        self.uiWic2Label.setObjectName("uiWic2Label")
        self.gridLayout_5.addWidget(self.uiWic2Label, 2, 0, 1, 1)
        self.uiWic2comboBox = QtWidgets.QComboBox(self.uiWicWizardPage)
        self.uiWic2comboBox.setObjectName("uiWic2comboBox")
        self.gridLayout_5.addWidget(self.uiWic2comboBox, 2, 1, 1, 1)
        IOSRouterWizard.addPage(self.uiWicWizardPage)
        self.uiIdlePCWizardPage = QtWidgets.QWizardPage()
        self.uiIdlePCWizardPage.setObjectName("uiIdlePCWizardPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiIdlePCWizardPage)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiIdlepcLabel = QtWidgets.QLabel(self.uiIdlePCWizardPage)
        self.uiIdlepcLabel.setObjectName("uiIdlepcLabel")
        self.gridLayout_3.addWidget(self.uiIdlepcLabel, 0, 0, 1, 1)
        self.uiIdlepcLineEdit = QtWidgets.QLineEdit(self.uiIdlePCWizardPage)
        self.uiIdlepcLineEdit.setObjectName("uiIdlepcLineEdit")
        self.gridLayout_3.addWidget(self.uiIdlepcLineEdit, 0, 1, 1, 1)
        self.uiIdlePCFinderPushButton = QtWidgets.QPushButton(self.uiIdlePCWizardPage)
        self.uiIdlePCFinderPushButton.setObjectName("uiIdlePCFinderPushButton")
        self.gridLayout_3.addWidget(self.uiIdlePCFinderPushButton, 0, 2, 1, 1)
        IOSRouterWizard.addPage(self.uiIdlePCWizardPage)

        self.retranslateUi(IOSRouterWizard)
        QtCore.QMetaObject.connectSlotsByName(IOSRouterWizard)
        IOSRouterWizard.setTabOrder(self.uiNameLineEdit, self.uiPlatformComboBox)

    def retranslateUi(self, IOSRouterWizard):
        _translate = QtCore.QCoreApplication.translate
        IOSRouterWizard.setWindowTitle(_translate("IOSRouterWizard", "New IOS router template"))
        self.uiServerWizardPage.setTitle(_translate("IOSRouterWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please choose a server type to run your new IOS router."))
        self.uiServerTypeGroupBox.setTitle(_translate("IOSRouterWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("IOSRouterWizard", "Run the IOS on a remote computer"))
        self.uiVMRadioButton.setText(_translate("IOSRouterWizard", "Run the IOS on the GNS3 VM"))
        self.uiLocalRadioButton.setText(_translate("IOSRouterWizard", "Run the IOS on your local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("IOSRouterWizard", "Remote server"))
        self.uiRemoteServersLabel.setText(_translate("IOSRouterWizard", "Run on:"))
        self.uiIOSImageWizardPage.setTitle(_translate("IOSRouterWizard", "IOS image"))
        self.uiIOSImageWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please choose an IOS image."))
        self.uiIOSExistingImageRadioButton.setText(_translate("IOSRouterWizard", "Existing image"))
        self.uiIOSNewImageRadioButton.setText(_translate("IOSRouterWizard", "New Image"))
        self.uiIOSImageLabel.setText(_translate("IOSRouterWizard", "IOS image:"))
        self.uiIOSImageToolButton.setText(_translate("IOSRouterWizard", "&Browse..."))
        self.uiNamePlatformWizardPage.setTitle(_translate("IOSRouterWizard", "Name and platform"))
        self.uiNamePlatformWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please choose a descriptive name for this new IOS router and verify the platform and chassis."))
        self.uiTypeLabel.setText(_translate("IOSRouterWizard", "Platform:"))
        self.uiNameLabel.setText(_translate("IOSRouterWizard", "Name:"))
        self.uiChassisLabel.setText(_translate("IOSRouterWizard", "Chassis:"))
        self.uiEtherSwitchCheckBox.setText(_translate("IOSRouterWizard", "This is an EtherSwitch router"))
        self.uiMemoryWizardPage.setTitle(_translate("IOSRouterWizard", "Memory"))
        self.uiMemoryWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please check the amount of memory (RAM) that you allocate to IOS. Too much or not enough RAM could prevent IOS from starting."))
        self.uiRamLabel.setText(_translate("IOSRouterWizard", "Default RAM:"))
        self.uiRamSpinBox.setSuffix(_translate("IOSRouterWizard", " MiB"))
        self.uiTestIOSImagePushButton.setText(_translate("IOSRouterWizard", "&Test IOS image"))
        self.label.setText(_translate("IOSRouterWizard", "<html><head/><body><p><a href=\"http://tools.cisco.com/ITDIT/CFN/jsp/SearchBySoftware.jsp\"><span style=\" text-decoration: underline; color:#0000ff;\">Check for minimum and maximum RAM requirement</span></a></p></body></html>"))
        self.uiNetworkAdaptersWizardPage.setTitle(_translate("IOSRouterWizard", "Network adapters"))
        self.uiNetworkAdaptersWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please choose the default network adapters that should be inserted into every new instance of this router."))
        self.uiSlot0Label.setText(_translate("IOSRouterWizard", "slot 0:"))
        self.uiSlot1Label.setText(_translate("IOSRouterWizard", "slot 1:"))
        self.uiSlot2Label.setText(_translate("IOSRouterWizard", "slot 2:"))
        self.uiSlot3Label.setText(_translate("IOSRouterWizard", "slot 3:"))
        self.uiSlot4Label.setText(_translate("IOSRouterWizard", "slot 4:"))
        self.uiSlot5Label.setText(_translate("IOSRouterWizard", "slot 5:"))
        self.uiSlot6Label.setText(_translate("IOSRouterWizard", "slot 6:"))
        self.uiWicWizardPage.setTitle(_translate("IOSRouterWizard", "WIC modules"))
        self.uiWicWizardPage.setSubTitle(_translate("IOSRouterWizard", "Please choose the default WIC modules that should be inserted into every new instance of this router."))
        self.uiWic0Label.setText(_translate("IOSRouterWizard", "wic 0:"))
        self.uiWic1Label.setText(_translate("IOSRouterWizard", "wic 1:"))
        self.uiWic2Label.setText(_translate("IOSRouterWizard", "wic 2:"))
        self.uiIdlePCWizardPage.setTitle(_translate("IOSRouterWizard", "Idle-PC"))
        self.uiIdlePCWizardPage.setSubTitle(_translate("IOSRouterWizard", "An idle-pc value is necessary to prevent IOS to use 100% of your processor or one of its cores."))
        self.uiIdlepcLabel.setText(_translate("IOSRouterWizard", "Idle-PC:"))
        self.uiIdlePCFinderPushButton.setText(_translate("IOSRouterWizard", "Idle-PC finder"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/snapshots_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SnapshotsDialog(object):

    def setupUi(self, SnapshotsDialog):
        SnapshotsDialog.setObjectName("SnapshotsDialog")
        SnapshotsDialog.setWindowModality(QtCore.Qt.WindowModal)
        SnapshotsDialog.resize(496, 288)
        self.gridLayout = QtWidgets.QGridLayout(SnapshotsDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.uiSnapshotsList = QtWidgets.QListWidget(SnapshotsDialog)
        self.uiSnapshotsList.setObjectName("uiSnapshotsList")
        self.gridLayout.addWidget(self.uiSnapshotsList, 0, 0, 1, 4)
        self.uiCreatePushButton = QtWidgets.QPushButton(SnapshotsDialog)
        self.uiCreatePushButton.setObjectName("uiCreatePushButton")
        self.gridLayout.addWidget(self.uiCreatePushButton, 1, 0, 1, 1)
        self.uiRestorePushButton = QtWidgets.QPushButton(SnapshotsDialog)
        self.uiRestorePushButton.setEnabled(False)
        self.uiRestorePushButton.setObjectName("uiRestorePushButton")
        self.gridLayout.addWidget(self.uiRestorePushButton, 1, 2, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(SnapshotsDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout.addWidget(self.uiButtonBox, 1, 3, 1, 1)
        self.uiDeletePushButton = QtWidgets.QPushButton(SnapshotsDialog)
        self.uiDeletePushButton.setEnabled(False)
        self.uiDeletePushButton.setObjectName("uiDeletePushButton")
        self.gridLayout.addWidget(self.uiDeletePushButton, 1, 1, 1, 1)

        self.retranslateUi(SnapshotsDialog)
        self.uiButtonBox.accepted.connect(SnapshotsDialog.accept)
        self.uiButtonBox.rejected.connect(SnapshotsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SnapshotsDialog)

    def retranslateUi(self, SnapshotsDialog):
        _translate = QtCore.QCoreApplication.translate
        SnapshotsDialog.setWindowTitle(_translate("SnapshotsDialog", "Snapshots"))
        self.uiCreatePushButton.setText(_translate("SnapshotsDialog", "&Create"))
        self.uiRestorePushButton.setText(_translate("SnapshotsDialog", "&Restore"))
        self.uiDeletePushButton.setText(_translate("SnapshotsDialog", "&Delete"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from zipfile import ZipFile
import platform
import psutil
import os

from gns3.version import __version__
from gns3.qt import QtWidgets, QtCore
from gns3.ui.export_debug_dialog_ui import Ui_ExportDebugDialog
from gns3.local_config import LocalConfig

import logging
log = logging.getLogger(__name__)

class ExportDebugDialog(QtWidgets.QDialog, Ui_ExportDebugDialog):
    """
    This dialog allow user to export useful information
    for remote debugging by a GNS3 developers.
    """

    def __init__(self, parent, project):

        super().__init__(parent)
        self._project = project
        self.setupUi(self)
        self.uiOkButton.clicked.connect(self._okButtonClickedSlot)

    def _okButtonClickedSlot(self):
        path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Export debug file", None, "Zip file (*.zip)", "Zip file (*.zip)")

        if len(path) == 0:
            self.reject()
            return

        log.info("Export debug information to %s", path)

        try:
            with ZipFile(path, 'w') as zip:
                zip.writestr("debug.txt", self._getDebugData())
                dir = LocalConfig.configDirectory()
                for filename in os.listdir(dir):
                    path = os.path.join(dir, filename)
                    if os.path.isfile(path):
                        zip.write(path, filename)

                dir = self._project.filesDir()
                if dir:
                    for filename in os.listdir(dir):
                        path = os.path.join(dir, filename)
                        if os.path.isfile(path):
                            zip.write(path, filename)
        except OSError as e:
            QtWidgets.QMessageBox.critical(self, "Debug", "Can't export debug information: {}".format(str(e)))
        self.accept()

    def _getDebugData(self):
        try:
            connections = psutil.net_connections()
        # You need to be root for OSX
        except psutil.AccessDenied:
            connections = None

        try:
            addrs = ["* {}: {}".format(key, val) for key, val in psutil.net_if_addrs().items()]
        except UnicodeDecodeError:
            addrs = ["INVALID ADDR WITH UNICODE CHARACTERS"]

        data = """Version: {version}
OS: {os}
Python: {python}
Qt: {qt}
PyQt: {pyqt}
CPU: {cpu}
Memory: {memory}

Networks:
{addrs}

Open connections:
{connections}

Processus:
""".format(
            version=__version__,
            qt=QtCore.QT_VERSION_STR,
            pyqt=QtCore.PYQT_VERSION_STR,
            os=platform.platform(),
            python=platform.python_version(),
            memory=psutil.virtual_memory(),
            cpu=psutil.cpu_times(),
            connections=connections,
            addrs="\n".join(addrs)
        )
        for proc in psutil.process_iter():
            try:
                psinfo = proc.as_dict(attrs=["name", "exe"])
                data += "* {} {}\n".format(psinfo["name"], psinfo["exe"])
            except psutil.NoSuchProcess:
                pass
        return data

if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    print(ExportDebugDialog(None)._getDebugData())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Thread to wait for the GNS3 VM.
"""

import os
import socket
import subprocess

from ..qt import QtCore
from ..version import __version__
from ..gns3_vm import GNS3VM
from ..servers import Servers

import logging
log = logging.getLogger(__name__)


class WaitForVMWorker(QtCore.QObject):

    """
    Thread to wait for the GNS3 VM to be started.
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    def __init__(self):

        super().__init__()
        self._is_running = False
        self._vm = GNS3VM.instance()

        vm_settings = self._vm.settings()
        self._vmname = vm_settings["vmname"]
        self._vmx_path = vm_settings["vmx_path"]
        self._headless = vm_settings["headless"]
        self._virtualization = vm_settings["virtualization"]

    def _get_vbox_vm_state(self):
        """
        Returns the VM state (e.g. running, paused etc.)

        :returns: state (string)
        """

        result = self._vm.execute_vboxmanage("showvminfo", [self._vmname, "--machinereadable"])
        for info in result.splitlines():
            if '=' in info:
                name, value = info.split('=', 1)
                if name == "VMState":
                    return value.strip('"')
        return "unknown"

    def _look_for_interface(self, network_backend):
        """
        Look for an interface with a specific network backend.

        :returns: interface number or -1 if none is found
        """

        result = self._vm.execute_vboxmanage("showvminfo", [self._vmname, "--machinereadable"])
        interface = -1
        for info in result.splitlines():
            if '=' in info:
                name, value = info.split('=', 1)
                if name.startswith("nic") and value.strip('"') == network_backend:
                    try:
                        interface = int(name[3:])
                        break
                    except ValueError:
                        continue
        return interface

    def _look_for_vboxnet(self, interface_number):
        """
        Look for the VirtualBox network name associated with a host only interface.

        :returns: None or vboxnet name
        """

        result = self._vm.execute_vboxmanage("showvminfo", [self._vmname, "--machinereadable"])
        for info in result.splitlines():
            if '=' in info:
                name, value = info.split('=', 1)
                if name == "hostonlyadapter{}".format(interface_number):
                    return value.strip('"')
        return None

    def _check_dhcp_server(self, vboxnet):
        """
        Check if the DHCP server associated with a vboxnet is enabled.

        :param vboxnet: vboxnet name

        :returns: boolean
        """

        properties = self._vm.execute_vboxmanage("list", ["dhcpservers"])
        flag_dhcp_server_found = False
        for prop in properties.splitlines():
            try:
                name, value = prop.split(':', 1)
            except ValueError:
                continue
            if name.strip() == "NetworkName" and value.strip().endswith(vboxnet):
                flag_dhcp_server_found = True
            if flag_dhcp_server_found and name.strip() == "Enabled":
                if value.strip() == "Yes":
                    return True
        return False

    def _check_vbox_port_forwarding(self):
        """
        Checks if the NAT port forwarding rule exists.

        :returns: boolean
        """

        result = self._vm.execute_vboxmanage("showvminfo", [self._vmname, "--machinereadable"])
        for info in result.splitlines():
            if '=' in info:
                name, value = info.split('=', 1)
                if name.startswith("Forwarding") and value.strip('"').startswith("GNS3VM"):
                    return True
        return False

    def _waitForServer(self, vm_server, endpoint, retry=0):
        """
        Wait for a VM server to reply to a request.

        :param vm_server: The server instance
        :param retry: How many time we need to retry if server doesn't answer wait 1 second between test
        """

        json_data = []
        status = 0
        while retry >= 0:
            status, json_data = vm_server.getSynchronous(endpoint, timeout=3)
            if status != 0 or not self._is_running:
                break
            self.thread().sleep(1)
            retry -= 1
        return status, json_data

    def run(self):
        """
        Worker starting point.
        """

        vm_server = Servers.instance().vmServer()
        self._is_running = True
        if self._virtualization == "VMware":
            self._is_running = self._start_vmware(vm_server)
        elif self._virtualization == "VirtualBox":
            self._is_running = self._start_virtualbox(vm_server)

        if not self._is_running:
            return

        log.info("GNS3 VM is started and server is running on {}:{}".format(vm_server.host(), vm_server.port()))
        try:
            status, json_data = self._waitForServer(vm_server, "version", retry=40)
            if status == 401:
                self.error.emit("Wrong user or password for the GNS3 VM".format(status), True)
                return
            elif status != 200:
                if status == 0:
                    msg = "Could not connect to GNS3 server {}:{} (please check your firewall settings)".format(vm_server.host(),
                                                                                                                vm_server.port())
                else:
                    msg = "Server has replied with status code {} when retrieving version number".format(status)
                log.error(msg)
                self.error.emit(msg, True)
                return
            server_version = json_data["version"]
            if __version__ != server_version:
                # Just a warning: the HTTP code for the connection to the server will block if the version has a mismatch
                print("Client version {} differs with server version {} in the GNS3 VM, please upgrade the VM by selecting the Upgrade options in the VM menu.".format(__version__, server_version))
        except OSError as e:
            self.error.emit("Request error {}".format(e), True)
            return

        self.finished.emit()

    def _start_vmware(self, vm_server):
        """
        Handle a VMware based GNS3 VM
        """

        # check we have a valid VMX file path
        if not self._vmx_path:
            self.error.emit("GNS3 VM is not configured", True)
            return False
        if not os.path.exists(self._vmx_path):
            self.error.emit("VMware VMX file {} doesn't exist".format(self._vmx_path), True)
            return False

        try:
            # start the VM
            args = [self._vmx_path]
            if self._headless:
                args.extend(["nogui"])
            self._vm.execute_vmrun("start", args)
            self._vm.setRunning(True)

            # check if the VMware guest tools are installed
            vmware_tools_state = self._vm.execute_vmrun("checkToolsState", [self._vmx_path])
            if vmware_tools_state not in ("installed", "running"):
                self.error.emit("VMware tools are not installed in {}".format(self._vmname), True)
                return False

            if not self._is_running:
                return False

            # get the guest IP address (first adapter only)
            guest_ip_address = self._vm.execute_vmrun("getGuestIPAddress", [self._vmx_path, "-wait"], timeout=120)
            vm_server.setHost(guest_ip_address)
            log.info("GNS3 VM IP address set to {}".format(guest_ip_address))
        except OSError as e:
            self.error.emit("Could not execute vmrun: {}".format(e), True)
            return False
        except subprocess.SubprocessError as e:
            self.error.emit("Could not execute vmrun: {} with output '{}'\n\nMake sure the correct product (Fusion, Workstation or Player) is selected in Preferences / VMware".format(e, e.output.decode("utf-8", errors="ignore").strip()), True)
            return False
        except subprocess.TimeoutExpired:
            self.error.emit("vmrun timeout expired", True)
            return False
        return True

    def _start_virtualbox(self, vm_server):
        """
        Handle a VirtualBox based GNS3 VM
        """

        try:
            # get a NAT interface number
            nat_interface_number = self._look_for_interface("nat")
            if nat_interface_number < 0:
                self.error.emit("The GNS3 VM must have a NAT interface configured in order to start", True)
                return False

            hostonly_interface_number = self._look_for_interface("hostonly")
            if hostonly_interface_number < 0:
                self.error.emit("The GNS3 VM must have a host only interface configured in order to start", True)
                return False

            vboxnet = self._look_for_vboxnet(hostonly_interface_number)
            if vboxnet is None:
                self.error.emit("VirtualBox host-only network could not be found for interface {}".format(hostonly_interface_number), True)
                return False

            if not self._check_dhcp_server(vboxnet):
                self.error.emit("DHCP must be enabled on VirtualBox host-only network: {}".format(vboxnet), True)
                return False

            vm_state = self._get_vbox_vm_state()
            log.info('"{}" state is {}'.format(self._vmname, vm_state))
            if vm_state in ("poweroff", "saved"):
                # start the VM if it is not running
                args = [self._vmname]
                if self._headless:
                    args.extend(["--type", "headless"])
                self._vm.execute_vboxmanage("startvm", args)
            self._vm.setRunning(True)

            ip_address = "127.0.0.1"
            try:
                # get a random port on localhost
                with socket.socket() as s:
                    s.bind((ip_address, 0))
                    port = s.getsockname()[1]
            except OSError as e:
                self.error.emit("Error while getting random port: {}".format(e), True)
                return False

            if self._check_vbox_port_forwarding():
                # delete the GNS3VM NAT port forwarding rule if it exists
                log.debug("Removing GNS3VM NAT port forwarding rule from interface {}".format(nat_interface_number))
                self._vm.execute_vboxmanage("controlvm", [self._vmname, "natpf{}".format(nat_interface_number), "delete", "GNS3VM"])

            # add a GNS3VM NAT port forwarding rule to redirect 127.0.0.1 with random port to port 3080 in the VM
            log.debug("Adding GNS3VM NAT port forwarding rule with port {} to interface {}".format(port, nat_interface_number))
            self._vm.execute_vboxmanage("controlvm", [self._vmname, "natpf{}".format(nat_interface_number), "GNS3VM,tcp,{},{},,3080".format(ip_address, port)])

            if not self._is_running:
                return False

            original_port = vm_server.port()
            vm_server.setPort(port)
            vm_server.setHost(ip_address)
            # ask the server all a list of all its interfaces along with IP addresses
            status, json_data = self._waitForServer(vm_server, "interfaces", retry=120)
            if status == 401:
                self.error.emit("Wrong user or password for the GNS3 VM".format(status), True)
                return False
            if status != 200:
                msg = "Server {} has replied with status code {} when retrieving the network interfaces".format(vm_server.url(), status)
                log.error(msg)
                self.error.emit(msg, True)
                return False

            # find the ip address for the first hostonly interface
            hostonly_ip_address_found = False
            for interface in json_data:
                if "name" in interface and interface["name"] == "eth{}".format(hostonly_interface_number - 1):
                    if "ip_address" in interface:
                        vm_server.setHost(interface["ip_address"])
                        vm_server.setPort(original_port)
                        log.info("GNS3 VM IP address set to {}".format(interface["ip_address"]))
                        hostonly_ip_address_found = True
                        break

            if not hostonly_ip_address_found:
                self.error.emit("Not IP address could be found in the GNS3 VM for eth{}".format(hostonly_interface_number - 1), True)
                return False

        except OSError as e:
            self.error.emit("Could not execute VBoxManage: {}".format(e), True)
            return False
        except subprocess.SubprocessError as e:
            self.error.emit("Could not execute VBoxManage: {} with output '{}'".format(e, e.output.decode("utf-8", errors="ignore").strip()), True)
            return False
        except subprocess.TimeoutExpired:
            self.error.emit("VBoxmanage timeout expired", True)
            return False
        return True

    def cancel(self):
        """
        Cancel this worker.
        """

        if not self:
            return
        self._is_running = False
        self._vm.killRunningProcess()
        self._vm.setRunning(False)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/vmware/ui/vmware_vm_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VMwareVMWizard(object):
    def setupUi(self, VMwareVMWizard):
        VMwareVMWizard.setObjectName("VMwareVMWizard")
        VMwareVMWizard.resize(598, 453)
        VMwareVMWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiServerWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout.addWidget(self.uiRemoteRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout.addWidget(self.uiLocalRadioButton)
        self.gridLayout_2.addWidget(self.uiServerTypeGroupBox, 0, 0, 1, 1)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_8.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_8.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.uiRemoteServersGroupBox, 1, 0, 1, 1)
        VMwareVMWizard.addPage(self.uiServerWizardPage)
        self.uiVirtualBoxWizardPage = QtWidgets.QWizardPage()
        self.uiVirtualBoxWizardPage.setObjectName("uiVirtualBoxWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiVirtualBoxWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiVMListLabel = QtWidgets.QLabel(self.uiVirtualBoxWizardPage)
        self.uiVMListLabel.setObjectName("uiVMListLabel")
        self.gridLayout.addWidget(self.uiVMListLabel, 0, 0, 1, 1)
        self.uiVMListComboBox = QtWidgets.QComboBox(self.uiVirtualBoxWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMListComboBox.sizePolicy().hasHeightForWidth())
        self.uiVMListComboBox.setSizePolicy(sizePolicy)
        self.uiVMListComboBox.setObjectName("uiVMListComboBox")
        self.gridLayout.addWidget(self.uiVMListComboBox, 0, 1, 1, 1)
        self.uiBaseVMCheckBox = QtWidgets.QCheckBox(self.uiVirtualBoxWizardPage)
        self.uiBaseVMCheckBox.setEnabled(True)
        self.uiBaseVMCheckBox.setObjectName("uiBaseVMCheckBox")
        self.gridLayout.addWidget(self.uiBaseVMCheckBox, 1, 0, 1, 2)
        VMwareVMWizard.addPage(self.uiVirtualBoxWizardPage)

        self.retranslateUi(VMwareVMWizard)
        QtCore.QMetaObject.connectSlotsByName(VMwareVMWizard)

    def retranslateUi(self, VMwareVMWizard):
        _translate = QtCore.QCoreApplication.translate
        VMwareVMWizard.setWindowTitle(_translate("VMwareVMWizard", "New VMware VM template"))
        self.uiServerWizardPage.setTitle(_translate("VMwareVMWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("VMwareVMWizard", "Please choose a server type to run your new VMware VM."))
        self.uiServerTypeGroupBox.setTitle(_translate("VMwareVMWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("VMwareVMWizard", "Run the VMware VM on a remote computer"))
        self.uiLocalRadioButton.setText(_translate("VMwareVMWizard", "Run the VMware VM on your local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("VMwareVMWizard", "Remote servers"))
        self.uiRemoteServersLabel.setText(_translate("VMwareVMWizard", "Run on server:"))
        self.uiVirtualBoxWizardPage.setTitle(_translate("VMwareVMWizard", "VMware Virtual Machine"))
        self.uiVirtualBoxWizardPage.setSubTitle(_translate("VMwareVMWizard", "Please choose a VMware virtual machine from the list."))
        self.uiVMListLabel.setText(_translate("VMwareVMWizard", "VM list:"))
        self.uiBaseVMCheckBox.setText(_translate("VMwareVMWizard", "Use as a linked base VM (experimental)"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This module implements a QT4 Python interpreter widget.
It is inspired by PyCute : http://gerard.vermeulen.free.fr
"""

import sys
from .qt import QtCore, QtGui, QtWidgets


class MultipleRedirection(QtCore.QObject):
    writed = QtCore.pyqtSignal(str)

    def __init__(self, console, stdout):
        super().__init__()

        self.console = console
        self.stdout = stdout

    def write(self, str):
        self.writed.emit(str)
        if self.stdout.encoding is None:
            str = str.encode("ascii", "ignore").decode("ascii", "ignore")
        elif self.stdout.encoding != "UTF-8":
            str = str.encode(self.stdout.encoding, "ignore").decode(self.stdout.encoding, "ignore")
        self.stdout.write(str)

    def isatty(self):
        return self.console.isatty() or self.stdout.isatty()

    def flush(self):
        self.console.flush()
        try:
            self.stdout.flush()
        # On OSX when frozen flush raise a BrokenPipeError
        except BrokenPipeError:
            pass


class PyCutExt(QtWidgets.QTextEdit):

    """
    PyCute is a Python shell for PyQt.

    Creating, displaying and controlling PyQt widgets from the Python command
    line interpreter is very hard, if not, impossible.  PyCute solves this
    problem by interfacing the Python interpreter to a PyQt widget.
    """

    def __init__(self, interpreter, message="", log="", parent=None):

        super().__init__(parent)

        self.interpreter = interpreter
        self.colorizer = SyntaxColor()

        # session log
        self.log = log or ''

        # to exit the main interpreter by a Ctrl-D if PyCute has no parent
        if parent is None:
            self.eofKey = QtCore.Qt.Key_D
        else:
            self.eofKey = None

        # capture all interactive input/output
        self._old_stdout = sys.stdout
        sys.stdout = MultipleRedirection(self, sys.stdout)
        sys.stdout.writed.connect(self.write)
        # sys.stderr = MultipleRedirection((sys.stderr, self))
        self._old_stdin = sys.stdin
        sys.stdin = self

        # last line + last incomplete lines
        self.line = ""
        self.lines = []

        # the cursor position in the last line
        self.point = 0

        # flag: the interpreter needs more input to run the last lines.
        self.more = 0

        # flag: readline() is being used for e.g. raw_input() and input()
        self.reading = 0

        # history
        self.history = []
        self.pointer = 0
        self.cursor_pos = 0

        self.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)

        try:
            sys.ps1
        except AttributeError:
            sys.ps1 = ">>> "
        try:
            sys.ps2
        except AttributeError:
            sys.ps2 = "... "

        self.write(message + '\n\n')
        self.write(sys.ps1)

    def closeIO(self):
        """
        Call when parent is closing we need to stop writing in the windows
        and go back to the standard output
        """
        sys.stdout = self._old_stdout
        sys.stdin = self._old_stdin

    def get_interpreter(self):
        """
        Return the interpreter object
        """

        return self.interpreter

    def moveCursor(self, operation, mode=QtGui.QTextCursor.MoveAnchor):
        """
        Convenience function to move the cursor
        This function will be present in PyQT4.2
        """
        cursor = self.textCursor()
        cursor.movePosition(operation, mode)
        self.setTextCursor(cursor)

    def flush(self):
        """
        Simulate stdin, stdout, and stderr.
        """
        pass

    def isatty(self):
        """
        Simulate stdin, stdout, and stderr.
        """
        return 1

    def readline(self):
        """
        Simulate stdin, stdout, and stderr.
        """

        self.reading = 1
        self._clearLine()
        self.moveCursor(QtGui.QTextCursor.End)
        while self.reading:
            QtWidgets.QApplication.processEvents(QtCore.QEventLoop.AllEvents, 1000)
        if len(self.line) == 0:
            return '\n'
        else:
            return self.line

    def write(self, text, error=False, warning=False):
        """
        Simulates stdin, stdout, and stderr.
        """

        cursor = self.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)

        pos1 = cursor.position()
        cursor.insertText(text)

        self.cursor_pos = cursor.position()
        self.setTextCursor(cursor)
        self.ensureCursorVisible()

        # Set the format
        cursor.setPosition(pos1, QtGui.QTextCursor.KeepAnchor)
        char_format = cursor.charFormat()
        if error:
            color = QtGui.QColor(255, 0, 0)  # red
        elif warning:
            color = QtGui.QColor(255, 128, 0)  # orange
        else:
            color = QtGui.QColor(0, 0, 0)  # black
        char_format.setForeground(QtGui.QBrush(color))
        cursor.setCharFormat(char_format)

    def writelines(self, text):
        """
        Simulate stdin, stdout, and stderr.
        """
        map(self.write, text)

    def _run(self):
        """
        Append the last line to the history list, let the interpreter execute
        the last line(s), and clean up accounting for the interpreter results:
        (1) the interpreter succeeds
        (2) the interpreter fails, finds no errors and wants more line(s)
        (3) the interpreter fails, finds errors and writes them to sys.stderr
        """

        self.pointer = 0
        self.history.append(self.line)
        try:
            self.lines.append(self.line)
        except Exception as e:
            print(e)

        source = '\n'.join(self.lines)
        self.more = self.interpreter.runsource(source)

        if self.more:
            self.write(sys.ps2)
        else:
            self.write(sys.ps1)
            self.lines = []
        self._clearLine()

    def _clearLine(self):
        """
        Clear input line buffer
        """
        self.line = ""
        self.point = 0

    def _insertText(self, text):
        """
        Inserts text at the current cursor position.
        """

        self.line = self.line[:self.point] + text + self.line[self.point:]
        self.point += len(text)

        cursor = self.textCursor()
        cursor.insertText(text)
        self.color_line()

    def keyPressEvent(self, e):
        """
        Handle user input a key at a time.
        """

        text = e.text()
        key = e.key()

        if e.modifiers() == QtCore.Qt.ControlModifier:
            return super().keyPressEvent(e)

        # Keep the cursor after the last prompt.
        self.moveCursor(QtGui.QTextCursor.End)

        if key == QtCore.Qt.Key_Backspace:
            if self.point:
                cursor = self.textCursor()
                cursor.movePosition(QtGui.QTextCursor.PreviousCharacter, QtGui.QTextCursor.KeepAnchor)
                cursor.removeSelectedText()
                self.color_line()

                self.point -= 1
                self.line = self.line[:-1]

        elif key == QtCore.Qt.Key_Delete:
            cursor = self.textCursor()
            cursor.movePosition(QtGui.QTextCursor.NextCharacter, QtGui.QTextCursor.KeepAnchor)
            cursor.removeSelectedText()
            self.color_line()

            self.line = self.line[:-1]

        elif key == QtCore.Qt.Key_Return or key == QtCore.Qt.Key_Enter:
            self.write("\n")
            if self.reading:
                self.reading = 0
            else:
                self._run()

        elif key == QtCore.Qt.Key_Tab:
            self.onKeyPress_Tab()
        elif key == QtCore.Qt.Key_Left:
            if self.point:
                self.moveCursor(QtGui.QTextCursor.Left)
                self.point -= 1
        elif key == QtCore.Qt.Key_Right:
            if self.point < len(self.line):
                self.moveCursor(QtGui.QTextCursor.Right)
                self.point += 1

        elif key == QtCore.Qt.Key_Home:
            cursor = self.textCursor()
            cursor.setPosition(self.cursor_pos)
            self.setTextCursor(cursor)
            self.point = 0

        elif key == QtCore.Qt.Key_End:
            self.moveCursor(QtGui.QTextCursor.EndOfLine)
            self.point = len(self.line)

        elif key == QtCore.Qt.Key_Up:

            if len(self.history):
                if self.pointer == 0:
                    self.pointer = len(self.history)
                self.pointer -= 1
                self._recall()

        elif key == QtCore.Qt.Key_Down:
            if len(self.history):
                self.pointer += 1
                if self.pointer == len(self.history):
                    self.pointer = 0
                self._recall()

        elif len(text):
            self._insertText(text)
            return

    def _recall(self):
        """
        Display the current item from the command history.
        """

        cursor = self.textCursor()
        cursor.select(QtGui.QTextCursor.LineUnderCursor)
        cursor.removeSelectedText()

        if self.more:
            self.write(sys.ps2)
        else:
            self.write(sys.ps1)

        self._clearLine()
        self._insertText(self.history[self.pointer])

    def contentsContextMenuEvent(self, ev):
        """
        Suppress the right button context menu.
        """

        return

    def color_line(self):
        """
        Color the current line.
        """

        cursor = self.textCursor()
        cursor.movePosition(QtGui.QTextCursor.StartOfLine)

        newpos = cursor.position()
        pos = -1

        while (newpos != pos):
            cursor.movePosition(QtGui.QTextCursor.NextWord)

            pos = newpos
            newpos = cursor.position()

            cursor.select(QtGui.QTextCursor.WordUnderCursor)
            word = cursor.selectedText()

            if not word:
                continue

            (R, G, B) = self.colorizer.get_color(word)
            char_format = cursor.charFormat()
            char_format.setForeground(QtGui.QBrush(QtGui.QColor(R, G, B)))
            cursor.setCharFormat(char_format)


class SyntaxColor:

    """
    Allows to color python keywords.
    """

    keywords = set(["and", "del", "from", "not", "while",
                    "as", "elif", "global", "or", "with",
                    "assert", "else", "if", "pass", "yield",
                    "break", "except", "import", "print",
                    "class", "exec", "in", "raise",
                    "continue", "finally", "is", "return",
                    "def", "for", "lambda", "try"])

    def get_color(self, word):
        """ Return a color tuple (R,G,B) depending of the string word """

        stripped = word.strip()

        if(stripped in self.keywords):
            return (165, 42, 42)   # brown
        elif(self.is_python_string(stripped)):
            return (61, 120, 9)   # dark green
        else:
            return (0, 0, 0)

    def is_python_string(self, string):
        """
        Return True if string is enclosed by a string mark
        """

#         return (
#             (string.startswith("'''") and string.endswith("'''")) or
#             (string.startswith('"""') and string.endswith('"""')) or
#             (string.startswith("'") and string.endswith("'")) or
#             (string.startswith('"') and string.endswith('"'))
#             )
        return False
